;Procedure       : EXPLODEWINDOW(TROW,TCOL,BROW,BCOL,ATTRIB,SHADOW)
;Author          : Peter M. Jagielski
;Memory Size     : 4736 bytes
;Description     : This procedure writes an "exploding" window to the PAL
;                  canvas in a user specified color and size, with or without
;                  a shadow behind the window.  For size and speed sake, this
;                  procedure does no error checking for box corners which are
;                  out of bounds.  Valid rows are >=0 and <=24 and valid columns
;                  are >=0 and <= 79.
;Parameters      : trow = top left corner row
;                  tcol = top left corner column
;                  brow = bottom right corner row
;                  bcol = bottom right corner column
;                  attrib = color attribute (see Appendix A in PAL User's Guide)
;                  shadow = true or false
;Returns Value?  : no

proc explodewindow(trow,tcol,brow,bcol,attrib,shadow)
 private columnratio,      ;ratio/scale at which to expand columns, since there are more columns than rows
         r1,               ;contains beginning row and column corners
         c1,               ;
         r2,               ;
         c2                ;
  cursor off
  style attribute attrib                                      ;must set color for 4 corners that get placed at end
  columnratio = round(((bcol - tcol) / (brow - trow)) + 1,0)  ;get "scaling" ratio
  r1 = round((trow + brow) / 2,0)                             ;compute center row of box
  c1 = round((tcol + bcol) / 2,0)                             ;compute center column of box
  r2 = r1                                                     ;initialize opposite corners
  c2 = c1
  while (true)                                                ;while we haven't reached the corners yet
    if (r1 > trow) then
      r1 = r1 - 1
    endif
    if (r2 < brow) then
      r2 = r2 + 1
    endif
    if (c1 > tcol + columnratio) then
      c1 = c1 - columnratio
    else
      c1 = tcol
    endif
    if (c2 < bcol - columnratio) then
      c2 = c2 + columnratio
    else
      c2 = bcol
    endif
    paintcanvas fill " " attribute attrib r1,c1,r2,c2             ;place a colored box
    if (r1 = trow and c1 = tcol and r2 = brow and c2 = bcol) then ;has the full box been drawn yet?
      quitloop                                                    ;get out of this while loop
    endif
  endwhile
  if (shadow) then
    paintcanvas attribute 0 brow+1,tcol+2,brow+1,bcol+2           ;place horizontal shadow
    paintcanvas attribute 0 trow+1,bcol+1,brow,bcol+2             ;place vertical shadow
  endif
  paintcanvas fill "" attribute attrib trow,tcol+1,trow,bcol-1   ;place the box borders
  paintcanvas fill "" attribute attrib brow,tcol+1,brow,bcol-1
  paintcanvas fill "" attribute attrib trow+1,tcol,brow-1,tcol
  paintcanvas fill "" attribute attrib trow+1,bcol,brow-1,bcol
  @ trow,tcol ?? ""                                              ;place the box corners
  @ trow,bcol ?? ""
  @ brow,tcol ?? ""
  @ brow,bcol ?? ""
endproc
